@@ -4,73 +4,64 @@ Uses yt-dlp and ffmpeg with automatic binary management
 """
 
 import asyncio
-from asyncio.subprocess import Process
-import json
+from json import loads
 import os
-import platform
 import re
-import shutil
-import zipfile
 from pathlib import Path
-from typing import (
-    Any,
-    AsyncGenerator,
-    Awaitable,
-    Callable,
-    Dict,
-    List,
-    Optional,
-    Union,
-    overload,
-)
+import shutil
+from typing import Any, Awaitable, Callable, Dict, List, Optional, Union, overload
 from collections.abc import Callable as Callable2
-import aiofiles
-import aiohttp
 import logging
+import warnings
+import tempfile
 
-from asyncyt.exceptions import (
-    AsyncYTBase,
-    DownloadAlreadyExistsError,
-    DownloadGotCanceledError,
-    DownloadNotFoundError,
-    YtdlpDownloadError,
-    YtdlpGetInfoError,
-    YtdlpPlaylistGetInfoError,
-    YtdlpSearchError,
+from .enums import ProgressStatus
+from .exceptions import *
+from .basemodels import *
+from .utils import (
+    call_callback,
+    get_id,
+    get_unique_filename,
+    get_unique_path,
 )
-
-from .enums import AudioFormat, VideoFormat, Quality
-from .basemodels import (
-    DownloadFileProgress,
-    SetupProgress,
-    VideoInfo,
-    DownloadConfig,
-    DownloadProgress,
-    DownloadRequest,
-    SearchRequest,
-    PlaylistRequest,
-    DownloadResponse,
-    SearchResponse,
-    PlaylistResponse,
-    HealthResponse,
-)
-from .utils import call_callback, get_id, get_unique_filename
-from .binaries import BinaryManager
+from .binaries import AsyncFFmpeg
 
 logger = logging.getLogger(__name__)
 
-__all__ = ["Downloader"]
+__all__ = ["AsyncYT", "Downloader"]
 
 
-class Downloader(BinaryManager):
-    """Main downloader class with async support"""
+class AsyncYT(AsyncFFmpeg):
+    """
+    AsyncYT: Asynchronous YouTube Downloader and Searcher
 
-    def __init__(self):
-        super().__init__()
-        self._downloads: Dict[str, Process] = {}
+    This class provides asynchronous methods for downloading YouTube videos, playlists, and searching for videos using yt-dlp and FFmpeg. It supports progress tracking, flexible configuration, and API-friendly response formats.
+
+    :param bin_dir: Path to the directory containing yt-dlp and FFmpeg binaries.
+    :type bin_dir: Optional[str | Path]
+    """
+
+    def __init__(self, bin_dir: Optional[str | Path] = None):
+        """
+        Initialize the AsyncYT instance.
+
+        :param bin_dir: Directory path for binary files (yt-dlp, FFmpeg).
+        :type bin_dir: Optional[str | Path]
+        """
+
+        super().__init__(setup_only_ffmpeg=False, bin_dir=bin_dir)
 
     async def get_video_info(self, url: str) -> VideoInfo:
-        """Get video information without downloading"""
+        """
+        Asynchronously retrieve video information from a given URL using yt-dlp.
+
+        :param url: The URL of the video to retrieve information for.
+        :type url: str
+        :return: VideoInfo object containing the video's metadata.
+        :rtype: VideoInfo
+        :raises YtdlpGetInfoError: If yt-dlp fails to retrieve video information.
+        """
+
         cmd = [str(self.ytdlp_path), "--dump-json", "--no-warnings", url]
 
         process = await asyncio.create_subprocess_exec(
@@ -82,11 +73,22 @@ class Downloader(BinaryManager):
         if process.returncode != 0:
             raise YtdlpGetInfoError(url, process.returncode, stderr.decode())
 
-        data = json.loads(stdout.decode())
+        data = loads(stdout.decode())
         return VideoInfo.from_dict(data)
 
     async def _search(self, query: str, max_results: int = 10) -> List[VideoInfo]:
-        """Search for videos"""
+        """
+        Search for videos by query.
+
+        :param query: Search query string.
+        :type query: str
+        :param max_results: Maximum number of results to return.
+        :type max_results: int
+        :return: List of VideoInfo objects.
+        :rtype: List[VideoInfo]
+        :raises YtdlpSearchError: If yt-dlp search fails.
+        """
+
         search_url = f"ytsearch{max_results}:{query}"
 
         cmd = [str(self.ytdlp_path), "--dump-json", "--no-warnings", search_url]
@@ -103,7 +105,7 @@ class Downloader(BinaryManager):
         results = []
         for line in stdout.decode().strip().split("\n"):
             if line:
-                data = json.loads(line)
+                data = loads(line)
                 results.append(VideoInfo.from_dict(data))
 
         return results
@@ -120,6 +122,16 @@ class Downloader(BinaryManager):
             ],
         ],
     ):
+        """
+        Parse and validate download configuration arguments.
+
+        :param args: Positional arguments (url, config, progress_callback, DownloadRequest).
+        :param kwargs: Keyword arguments (url, config, progress_callback, request).
+        :return: Tuple of (url, config, progress_callback)
+        :rtype: Tuple[str, Optional[DownloadConfig], Optional[Callable]]
+        :raises TypeError: If arguments are invalid.
+        """
+
         url: Optional[str] = None
         config: Optional[DownloadConfig] = None
         progress_callback: Optional[
@@ -167,6 +179,7 @@ class Downloader(BinaryManager):
             Callable[[DownloadProgress], Union[None, Awaitable[None]]]
         ] = None,
     ) -> str: ...
+
     @overload
     async def download(
         self,
@@ -177,10 +190,24 @@ class Downloader(BinaryManager):
     ) -> str: ...
 
     async def download(self, *args, **kwargs) -> str:
-        """Download a video with the given configuration"""
+        """
+        Asynchronously download media from a given URL using yt-dlp, track progress, and process the output file.
+
+        :param args: url (str) or request (DownloadRequest)
+        :param kwargs: config (Optional[DownloadConfig]), progress_callback (Optional[Callable])
+        :return: The filename of the output File.
+        :rtype: str
+        :raises DownloadAlreadyExistsError: If a download with the same ID is already in progress.
+        :raises YtdlpDownloadError: If yt-dlp returns a non-zero exit code.
+        :raises Exception: If the output file cannot be determined from yt-dlp output.
+        :raises DownloadGotCanceledError: If the download is cancelled.
+        :raises FileNotFoundError: If FFmpeg wasn't installed.
+        """
+
         url, config, progress_callback = self._get_config(*args, **kwargs)
         if not config:
             config = DownloadConfig()
+
         id = get_id(url, config)
         if id in self._downloads:
             raise DownloadAlreadyExistsError(id)
@@ -188,6 +215,20 @@ class Downloader(BinaryManager):
         # Ensure output directory exists
         output_dir = Path(config.output_path)
         output_dir.mkdir(parents=True, exist_ok=True)
+        temp_dir = tempfile.TemporaryDirectory()
+        temp_path = Path(temp_dir.name)
+        logger.debug(temp_path)
+        config.output_path = str(temp_path.absolute())
+
+        if not self.ffmpeg_path:
+            raise FileNotFoundError("FFmpeg isn't installed")
+
+        config.ffmpeg_config.ffmpeg_path = str(self.ffmpeg_path)
+        _original_embed_thumbnail = config.embed_thumbnail
+        _original_write_thumbnail = config.write_thumbnail
+        if config.embed_thumbnail:
+            config.embed_thumbnail = False
+            config.write_thumbnail = True
 
         # Build yt-dlp command
         cmd = await self._build_download_command(url, config)
@@ -195,49 +236,55 @@ class Downloader(BinaryManager):
         # Create progress tracker
         progress = DownloadProgress(url=url, percentage=0, id=id)
 
-        # Execute download
-        process = await asyncio.create_subprocess_exec(
-            *cmd,
-            stdout=asyncio.subprocess.PIPE,
-            stderr=asyncio.subprocess.STDOUT,
-            cwd=output_dir,
-        )
+        output_file: Optional[str] = None  # Initialize properly
 
-        self._downloads[id] = process
-
-        output_file: Optional[str] = None
-
-        output: List[str] = []
-        # Monitor progress
-        async for line in self._read_process_output(process):
-            line = line.strip()
-            output.append(line)
-
-            if line:
-                old_percentage = progress.percentage
-                self._parse_progress(line, progress)
-
-                if progress_callback and progress.percentage > old_percentage:
-                    await call_callback(progress_callback, progress)
-
-                # Robust output filename extraction: only set if line is an absolute path, has a valid extension, and file exists
-                valid_exts = (
-                    ".mp3",
-                    ".m4a",
-                    ".wav",
-                    ".flac",
-                    ".ogg",
-                    ".wav",
-                    ".mp4",
-                    ".webm",
-                    ".mkv",
-                    ".avi",
-                    ".opus",
-                    ".aac",
-                )
-                if not output_file and line.lower().endswith(valid_exts):
-                    output_file = line
         try:
+            # Execute download
+            process = await asyncio.create_subprocess_exec(
+                *cmd,
+                stdout=asyncio.subprocess.PIPE,
+                stderr=asyncio.subprocess.STDOUT,
+                cwd=temp_path,
+            )
+
+            self._downloads[id] = process
+            output: List[str] = []
+
+            # Monitor yt-dlp progress
+            async for line in self._read_process_output(process):
+                line = line.strip()
+                output.append(line)
+
+                if line:
+                    old_percentage = progress.percentage
+                    self._parse_progress(line, progress)
+
+                    if progress_callback and progress.percentage > old_percentage:
+                        await call_callback(progress_callback, progress)
+
+                    # Robust output filename extraction
+                    valid_exts = (
+                        # Audio
+                        ".mp3",
+                        ".m4a",
+                        ".wav",
+                        ".flac",
+                        ".ogg",
+                        ".opus",
+                        ".aac",
+                        # Video
+                        ".mp4",
+                        ".webm",
+                        ".mkv",
+                        ".avi",
+                        ".flv",
+                        ".mov",
+                    )
+                    if not output_file and line.lower().endswith(valid_exts):
+                        # Verify the file actually exists and is an absolute path
+                        if os.path.isabs(line) and os.path.exists(line):
+                            output_file = line
+
             returncode = await process.wait()
 
             if returncode != 0:
@@ -245,20 +292,67 @@ class Downloader(BinaryManager):
                     url=url, output=output, cmd=cmd, error_code=returncode
                 )
 
+            if not output_file:
+                raise Exception("Could not determine output file from yt-dlp")
+
             if progress_callback:
-                progress.status = "finished"
+                progress.status = ProgressStatus.DOWNLOADED
                 progress.percentage = 100.0
                 await call_callback(progress_callback, progress)
-            return output_file  # type: ignore
+
+            config.embed_thumbnail = _original_embed_thumbnail
+            config.write_thumbnail = _original_write_thumbnail
+
+            result = await self.process(
+                output_file,
+                config.ffmpeg_config,
+                config,
+                progress_callback,
+                progress,
+                url,
+            )
+
+            config.output_path = str(output_dir)
+
+            for item in temp_path.iterdir():
+                dest_path = output_dir / item.name
+
+                if dest_path.exists():
+                    if config.ffmpeg_config.overwrite:
+                        destination = dest_path
+                    else:
+                        destination = get_unique_path(output_dir, item.name)
+                        destination = Path(destination)
+                else:
+                    destination = dest_path
+
+                try:
+                    shutil.move(str(item), str(destination))
+                except Exception as e:
+                    logger.error(f"Failed to move {item} to {destination}: {e}")
+
+            temp_dir.cleanup()
+
+            return result
+
         except asyncio.CancelledError:
-            process.kill()
-            await process.wait()
+            if id in self._downloads:
+                process = self._downloads[id]
+                process.kill()
+                await process.wait()
             raise DownloadGotCanceledError(id)
         finally:
             self._downloads.pop(id, None)
 
     async def cancel(self, download_id: str):
-        """cancel the downloading with download_id"""
+        """
+        Cancel the downloading with download_id.
+
+        :param download_id: The ID of the download to cancel.
+        :type download_id: str
+        :raises DownloadNotFoundError: If the download ID is not found.
+        """
+
         process = self._downloads.pop(download_id, None)
         if not process:
             raise DownloadNotFoundError(download_id)
@@ -284,7 +378,15 @@ class Downloader(BinaryManager):
     ) -> DownloadResponse: ...
 
     async def download_with_response(self, *args, **kwargs) -> DownloadResponse:
-        """Download with API-friendly response format"""
+        """
+        Download with API-friendly response format.
+
+        :param args: url (str) or request (DownloadRequest)
+        :param kwargs: config (Optional[DownloadConfig]), progress_callback (Optional[Callable])
+        :return: DownloadResponse object with metadata and error info.
+        :rtype: DownloadResponse
+        """
+
         try:
             url, config, progress_callback = self._get_config(*args, **kwargs)
             config = config or DownloadConfig()
@@ -332,27 +434,41 @@ class Downloader(BinaryManager):
 
     @overload
     async def search(
-        self, query: str, *, max_results: int, request: None = None
+        self, query: str, max_results: Optional[int] = None
     ) -> "SearchResponse": ...
 
     @overload
-    async def search(
-        self, query: None, *, max_results: None = None, request: "SearchRequest"
-    ) -> "SearchResponse": ...
+    async def search(self, *, request: "SearchRequest") -> "SearchResponse": ...
 
     async def search(
         self,
-        query: Optional[str],
-        *,
+        query: Optional[str] = None,
         max_results: Optional[int] = None,
+        *,
         request: Optional["SearchRequest"] = None,
     ) -> SearchResponse:
-        """Search with API-friendly response format"""
+        """
+        Perform an asynchronous search operation.
 
-        if (request is None and (query is None or max_results is None)) or (
-            request is not None and (query is not None or max_results is not None)
-        ):
-            raise TypeError("Choose either (query and max_results) or request.")
+        :param query: The search query string. Required if `request` is not provided.
+        :type query: Optional[str]
+        :param max_results: Maximum number of results to return. Defaults to 10.
+        :type max_results: Optional[int]
+        :param request: Optional SearchRequest object containing search parameters.
+        :type request: Optional[SearchRequest]
+        :return: SearchResponse object with results and status.
+        :rtype: SearchResponse
+        :raises TypeError: If both `request` and either `query` or `max_results` are provided, or if neither is provided.
+        """
+
+        if request is not None:
+            if query is not None or max_results is not None:
+                raise TypeError(
+                    "If you provide request, you cannot provide query, or max_results."
+                )
+        else:
+            if query is None:
+                raise TypeError("You must provide query when request is not given.")
 
         if request:
             query = request.query
@@ -377,10 +493,11 @@ class Downloader(BinaryManager):
     async def download_playlist(
         self,
         url: str,
-        *,
-        config: Optional[DownloadConfig],
-        max_videos: Optional[int],
-        progress_callback: Optional[Callable[[DownloadProgress], None]] = None,
+        config: Optional[DownloadConfig] = None,
+        max_videos: Optional[int] = None,
+        progress_callback: Optional[
+            Callable[[DownloadProgress], Union[None, Awaitable[None]]]
+        ] = None,
     ) -> PlaylistResponse: ...
 
     @overload
@@ -388,26 +505,49 @@ class Downloader(BinaryManager):
         self,
         *,
         request: PlaylistRequest,
-        progress_callback: Optional[Callable[[DownloadProgress], None]] = None,
+        progress_callback: Optional[
+            Callable[[DownloadProgress], Union[None, Awaitable[None]]]
+        ] = None,
     ) -> PlaylistResponse: ...
 
     async def download_playlist(
         self,
         url: Optional[str] = None,
-        *,
         config: Optional[DownloadConfig] = None,
         max_videos: Optional[int] = None,
-        progress_callback: Optional[Callable[[DownloadProgress], None]] = None,
+        progress_callback: Optional[
+            Callable[[DownloadProgress], Union[None, Awaitable[None]]]
+        ] = None,
         request: Optional[PlaylistRequest] = None,
     ) -> PlaylistResponse:
-        """Download playlist with API-friendly response format"""
-        if (
-            request is None and (url is None or config is None or max_videos is None)
-        ) or (
-            request is not None
-            and (url is not None or config is not None or max_videos is not None)
-        ):
-            raise TypeError("Choose either (url, config, and max_videos) or request.")
+        """
+        Asynchronously download videos from a YouTube playlist.
+
+        You can provide either a `request` object containing all parameters, or specify `url`, `config`, and `max_videos` individually. If `request` is provided, you must not provide `url`, `config`, or `max_videos`.
+
+        :param url: The URL of the playlist to download. Required if `request` is not given.
+        :type url: Optional[str]
+        :param config: Download configuration options.
+        :type config: Optional[DownloadConfig]
+        :param max_videos: Maximum number of videos to download from the playlist. Defaults to 100.
+        :type max_videos: Optional[int]
+        :param progress_callback: Optional callback to report download progress.
+        :type progress_callback: Optional[Callable[[DownloadProgress], Union[None, Awaitable[None]]]]
+        :param request: An object containing all playlist download parameters.
+        :type request: Optional[PlaylistRequest]
+        :return: PlaylistResponse object with download results.
+        :rtype: PlaylistResponse
+        :raises TypeError: If both `request` and any of `url`, `config`, or `max_videos` are provided, or if neither is provided.
+        """
+
+        if request is not None:
+            if url is not None or config is not None or max_videos is not None:
+                raise TypeError(
+                    "If you provide request, you cannot provide url, config, or max_videos."
+                )
+        else:
+            if url is None:
+                raise TypeError("You must provide url when request is not given.")
 
         if request:
             url = request.url
@@ -466,7 +606,16 @@ class Downloader(BinaryManager):
             )
 
     async def get_playlist_info(self, url: str) -> Dict[str, Any]:
-        """Get playlist information"""
+        """
+        Asynchronously retrieve information about a YouTube playlist using yt-dlp.
+
+        :param url: The URL of the YouTube playlist.
+        :type url: str
+        :return: Dictionary containing playlist entries and title.
+        :rtype: Dict[str, Any]
+        :raises YtdlpPlaylistGetInfoError: If the yt-dlp process fails to retrieve playlist information.
+        """
+
         cmd = [
             str(self.ytdlp_path),
             "--dump-json",
@@ -487,7 +636,7 @@ class Downloader(BinaryManager):
         entries = []
         for line in stdout.decode().strip().split("\n"):
             if line:
-                entries.append(json.loads(line))
+                entries.append(loads(line))
 
         return {
             "entries": entries,
@@ -497,5 +646,28 @@ class Downloader(BinaryManager):
                 else "Empty Playlist"
             ),
         }
-    #TODO: Add Cancel Function for playlist 
-    #TODO: also add basemodel for just the playlist downloading and the get_playlist_info
\ No newline at end of file
+
+    # TODO: also add basemodel for just the playlist downloading and the get_playlist_info
+
+
+class DeprecatedDownloader(AsyncYT):
+    """
+    .. deprecated::
+        Use :class:`AsyncYT` instead. This class will be removed in a future release.
+    """
+
+    def __init__(self, bin_dir: Optional[str | Path] = None):
+        """
+        .. deprecated::
+            Use :class:`AsyncYT` instead. This class will be removed in a future release.
+        """
+        warnings.warn(
+            "Downloader is deprecated and will be removed in a future release. "
+            "Please use AsyncYT instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        super().__init__(bin_dir)
+
+
+Downloader = DeprecatedDownloader
